(* Mathematica package *)
(* Created by the Wolfram Workbench 14.04.2016 *)


BeginPackage[
	"QuantumAlgebra`QuantumNotation`",
	{
		"QuantumAlgebra`",
		"QuantumAlgebra`Agreements`",
		"Notation`"
	}
]

(* ::Section:: *)
(* Exported symbols added here with SymbolName::usage *) 

SetQuantumNotation::usage =
	"SetQuantumNotation[] is used to permit input in Dirac Bra-Ket-Notation."

SetQuantumAliases::usage =
	"SetQuantumAliases[] sets keyboard aliases in the " <>
	"selected notebook. \nSetQuantumAliases[notebook] " <>
	"sets keyboard aliases in the specified notebook. " <>
	"The quantum aliases are keyboard key-combinations " <>
	"for the input of quantum objects in Dirac Notation."

SetQuantumAliases::aliases =
	"ALIASES:\n"<>
	"ESC op(du) ESC - Operator with or without indices\n"<>
	"ESC hop(du) ESC - Hermitian Operator with or without indices\n"<>
	"ESC ket(du) ESC - Ket with or without indices\n"<>
	"ESC (du)bra ESC - Bra with or without indices\n"<>
	"ESC (du)braket(du) ESC - BraKet with (left and / or right) subscript(s) and / or superscript(s).\n"<>
	"Letters in parenthesis signify optional indices (u - up, d - down) and must be given in that order.\n"<>
	"Example: use opd for an Operator with lower index."

SetNotationAliases::usage =
	"SetNotationAliases[nb] does the same as SetQuantumAliases[nb], see there."

SetBoxes::usage = 
	"SetBoxes[] is used to define proper structure of input / output boxes."

BraArgs::usage =
	"BraArgs is a tag for TagBoxes that guide the interpretation " <> 
	"of the internal structure of bra notation."

KetArgs::usage =
	"KetArgs is a tag for TagBoxes that guide the interpretation " <> 
	"of the internal structure of ket notation."

BraKetArgs::usage =
	"BraKetArgs is a tag for TagBoxes that guide the interpretation" <> 
	"of the internal structure of braket notation."

UDScript::usage =
	"UDScript is a tag for TagBoxes that guide the interpretation " <>  
	"of the internal structure of bras, kets, and brakets notation."

CloseNotationPalette::usage = "Closes the Notation Palette Notebook."

ScriptConvert::usage = "Converts scripted input (SuperscriptBox) to target expression."

ScriptConvert2::usage = "Converts scripted input (SubsuperscriptBox) to target expression."


Begin["`Private`"]

(* ::Section:: *)
(* Implementation of the package *)

(* ::Subsection:: *)
(* Input Conversions *)

(* Converts SuperscriptBox *)
ScriptConvert[x_, n_] := 
	Piecewise[
		{
			{ SuperDagger[x], ToString[n] == "\[Dagger]" },
			{ Power[x, n], NumericQ[x] || ! FreeQ[x, QuantumAlgebra`RulesSupplement`QASimplescript] || MemberQ[QAGetNotScriptedSymbols[], x] }
		},
		QuantumAlgebra`RulesSupplement`QAGetSimplescript[x, n, True]		
	]

ScriptConvert[x_, m_, n__] := QuantumAlgebra`RulesSupplement`QASubsuperscript[x, {}, {m, n}]


(* Converts SubsuperscriptBox *)
ScriptConvert2[x_, { m_Integer }, { n_Integer }] := QuantumAlgebra`RulesSupplement`QAGetSimplescript[x, m, False] ^ n

ScriptConvert2[x_, { m__ }, { n__ }] := QuantumAlgebra`RulesSupplement`QASubsuperscript[x, { m }, { n }]


(* ::Subsection:: *)
(* Quantum Notations *)

SetQuantumNotation[doc_NotebookObject] := 
	Module[{seq},
		seq = List[
			"NotationTemplateTag",
			BaseStyle->"NotationTemplateStyle",
			Editable->True
		];

		(* Ket Definitions *)
		Notation[
			ParsedBoxWrapper[ketBox["\[Psi]__"]] 
			\[DoubleLongLeftRightArrow] 
			ParsedBoxWrapper[toBoxes[Ket["\[Psi]__"]]]
		];

		Notation[
			ParsedBoxWrapper[ketBox["\[Psi]__", ketIndexBox[#1, SubscriptBox, "x__"]&]] 
			\[DoubleLongLeftRightArrow] 
			ParsedBoxWrapper[toBoxes[Ket["\[Psi]__", {"x__"}, {}]]]
		];

		Notation[
			ParsedBoxWrapper[ketBox["\[Psi]__", ketIndexBox[#1, SuperscriptBox, "x__"]&]]
			\[DoubleLongLeftRightArrow] 
			ParsedBoxWrapper[toBoxes[Ket["\[Psi]__", {}, {"x__"}]]]
		];

		Notation[
			ParsedBoxWrapper[ketBox["\[Psi]__", ketIndexBox[#1, SubsuperscriptBox, "x__", "y__"]&]]
			\[DoubleLongLeftRightArrow] 
			ParsedBoxWrapper[toBoxes[Ket["\[Psi]__", { "x__" }, { "y__" }]]]
		];

		(* Bra Definitions *)
		Notation[
			ParsedBoxWrapper[braBox["\[Psi]__"]] 
			\[DoubleLongLeftRightArrow] 
			ParsedBoxWrapper[toBoxes[Bra["\[Psi]__"]]]
		];

		Notation[
			ParsedBoxWrapper[braBox["\[Psi]__", braIndexBox[#1, SubscriptBox, "x__"]&]] 
			\[DoubleLongLeftRightArrow] 
			ParsedBoxWrapper[toBoxes[Bra["\[Psi]__", {"x__"}, {}]]]
		];

		Notation[
			ParsedBoxWrapper[braBox["\[Psi]__", braIndexBox[#1, SuperscriptBox, "x__"]&]]
			\[DoubleLongLeftRightArrow] 
			ParsedBoxWrapper[toBoxes[Bra["\[Psi]__", {}, {"x__"}]]]
		];

		Notation[
			ParsedBoxWrapper[braBox["\[Psi]__", braIndexBox[#1, SubsuperscriptBox, "x__", "y__"]&]]
			\[DoubleLongLeftRightArrow] 
			ParsedBoxWrapper[toBoxes[Bra["\[Psi]__", { "x__" }, { "y__" }]]]
		];

		(* BraKet Definitions *)
		Notation[
			ParsedBoxWrapper[braKetBox["\[Psi]__", "\[Phi]__"]]			
			\[DoubleLongLeftRightArrow]
			ParsedBoxWrapper[toBoxes[BraKet[{"\[Psi]__"}, {"\[Phi]__"}]]]
		];
		
		Notation[
			ParsedBoxWrapper[braKetBox["\[Psi]__", "\[Phi]__", ketIndexBox[#1, SubscriptBox, "x1__"]&]]
			\[DoubleLongLeftRightArrow] 
			ParsedBoxWrapper[toBoxes[BraKet[{"\[Psi]__"}, {"\[Phi]__", {"x1__"}, {}}]]]
		];

		Notation[
			ParsedBoxWrapper[braKetBox["\[Psi]__", "\[Phi]__", ketIndexBox[#1, SuperscriptBox, "y1__"]&]]
			\[DoubleLongLeftRightArrow] 
			ParsedBoxWrapper[toBoxes[BraKet[{"\[Psi]__"}, {"\[Phi]__", {}, {"y1__"}}]]]
		];

		Notation[
			ParsedBoxWrapper[braKetBox["\[Psi]__", "\[Phi]__", ketIndexBox[#1, SubsuperscriptBox, "x1__", "y1__"]&]]
			\[DoubleLongLeftRightArrow] 
			ParsedBoxWrapper[toBoxes[BraKet[{"\[Psi]__"}, {"\[Phi]__", {"x1__"}, {"y1__"}}]]]
		];
		
		Notation[
			ParsedBoxWrapper[braKetBox["\[Psi]__", "\[Phi]__", braIndexBox[#1, SubscriptBox, "x2__"]&]]
			\[DoubleLongLeftRightArrow] 
			ParsedBoxWrapper[toBoxes[BraKet[{"\[Psi]__", {"x2__"}, {}}, {"\[Phi]__"}]]]
		];
		
		Notation[
			ParsedBoxWrapper[braKetBox["\[Psi]__", "\[Phi]__", braIndexBox[#1, SuperscriptBox, "y2__"]&]]
			\[DoubleLongLeftRightArrow] 
			ParsedBoxWrapper[toBoxes[BraKet[{"\[Psi]__", {}, {"y2__"}}, {"\[Phi]__"}]]]
		];
		
		Notation[
			ParsedBoxWrapper[braKetBox["\[Psi]__", "\[Phi]__", braIndexBox[#1, SubsuperscriptBox, "x2__", "y2__"]&]]
			\[DoubleLongLeftRightArrow] 
			ParsedBoxWrapper[toBoxes[BraKet[{"\[Psi]__", {"x2__"}, {"y2__"}}, {"\[Phi]__"}]]]
		];

		Notation[
			ParsedBoxWrapper[braKetBox["\[Psi]__", "\[Phi]__", 
				braIndexBox[#1, SubscriptBox, "x2__"]&, 
				ketIndexBox[#1, SubscriptBox, "x1__"]&]]
			\[DoubleLongLeftRightArrow]
			ParsedBoxWrapper[toBoxes[BraKet[{"\[Psi]__", { "x2__" }, {}}, {"\[Phi]__", { "x1__" }, {}}]]]
		];

		Notation[
			ParsedBoxWrapper[braKetBox["\[Psi]__", "\[Phi]__", 
				braIndexBox[#1, SubscriptBox, "x2__"]&, 
				ketIndexBox[#1, SuperscriptBox, "y1__"]&]]
			\[DoubleLongLeftRightArrow]
			ParsedBoxWrapper[toBoxes[BraKet[{"\[Psi]__", { "x2__" }, {}}, {"\[Phi]__", {}, { "y1__" }}]]]
		];
		
		Notation[
			ParsedBoxWrapper[braKetBox["\[Psi]__", "\[Phi]__", 
				braIndexBox[#1, SubscriptBox, "x2__"]&, 
				ketIndexBox[#1, SubsuperscriptBox, "x1__", "y1__"]&]]
			\[DoubleLongLeftRightArrow]
			ParsedBoxWrapper[toBoxes[BraKet[{"\[Psi]__", { "x2__" }, {}}, {"\[Phi]__", { "x1__" }, { "y1__" }}]]]
		];
		
		Notation[
			ParsedBoxWrapper[braKetBox["\[Psi]__", "\[Phi]__", 
				braIndexBox[#1, SuperscriptBox, "y2__"]&, 
				ketIndexBox[#1, SubscriptBox, "x1__"]&]]
			\[DoubleLongLeftRightArrow]
			ParsedBoxWrapper[toBoxes[BraKet[{"\[Psi]__", {}, { "y2__" }}, {"\[Phi]__", { "x1__" }, {}}]]]
		];
		
		Notation[
			ParsedBoxWrapper[braKetBox["\[Psi]__", "\[Phi]__", 
				braIndexBox[#1, SuperscriptBox, "y2__"]&, 
				ketIndexBox[#1, SuperscriptBox, "y1__"]&]]
			\[DoubleLongLeftRightArrow]
			ParsedBoxWrapper[toBoxes[BraKet[{"\[Psi]__", {}, { "y2__" }}, {"\[Phi]__", {}, { "y1__" }}]]]
		];
		
		Notation[
			ParsedBoxWrapper[braKetBox["\[Psi]__", "\[Phi]__", 
				braIndexBox[#1, SuperscriptBox, "y2__"]&, 
				ketIndexBox[#1, SubsuperscriptBox, "x1__", "y1__"]&]]
			\[DoubleLongLeftRightArrow]
			ParsedBoxWrapper[toBoxes[BraKet[{"\[Psi]__", {}, { "y2__" }}, {"\[Phi]__", { "x1__" }, { "y1__" }}]]]
		];
		
		Notation[
			ParsedBoxWrapper[braKetBox["\[Psi]__", "\[Phi]__", 
				braIndexBox[#1, SubsuperscriptBox, "x2__", "y2__"]&, 
				ketIndexBox[#1, SubscriptBox, "x1__"]&]]
			\[DoubleLongLeftRightArrow]
			ParsedBoxWrapper[toBoxes[BraKet[{"\[Psi]__", { "x2__" }, { "y2__" }}, {"\[Phi]__", { "x1__" }, {}}]]]
		];
		
		Notation[
			ParsedBoxWrapper[braKetBox["\[Psi]__", "\[Phi]__", 
				braIndexBox[#1, SubsuperscriptBox, "x2__", "y2__"]&, 
				ketIndexBox[#1, SuperscriptBox, "y1__"]&]]
			\[DoubleLongLeftRightArrow]
			ParsedBoxWrapper[toBoxes[BraKet[{"\[Psi]__", { "x2__" }, { "y2__" }}, {"\[Phi]__", {}, { "y1__" }}]]]
		];
		
		Notation[
			ParsedBoxWrapper[braKetBox["\[Psi]__", "\[Phi]__", 
				braIndexBox[#1, SubsuperscriptBox, "x2__",  "y2__"]&, 
				ketIndexBox[#1, SubsuperscriptBox, "x1__", "y1__"]&]]
			\[DoubleLongLeftRightArrow]
			ParsedBoxWrapper[toBoxes[BraKet[{"\[Psi]__", { "x2__" }, { "y2__" }}, {"\[Phi]__", { "x1__" }, { "y1__" }}]]]
		];

		(* Operator Definitions *)
		Notation[		
			ParsedBoxWrapper[operatorBox["A_"]] 
			\[DoubleLongLeftRightArrow]
			ParsedBoxWrapper[toBoxes[Operator["A_"]]]
		];
			
		Notation[
			ParsedBoxWrapper[operatorBox["x_", operatorIndexBox[#1, "m__", ""]&]] 
			\[DoubleLongLeftRightArrow]
			ParsedBoxWrapper[toBoxes[Operator["x_", { "m__" }, {}]]]
		];

		Notation[
			ParsedBoxWrapper[operatorBox["x_", operatorIndexBox[#1, "", "n__"]&]] 
			\[DoubleLongLeftRightArrow]
			ParsedBoxWrapper[toBoxes[Operator["x_", {}, { "n__" }]]]
		];
		
		Notation[
			ParsedBoxWrapper[operatorBox["x_", operatorIndexBox[#1, "m__", "n__"]&]] 
			\[DoubleLongLeftRightArrow]
			ParsedBoxWrapper[toBoxes[Operator["x_", { "m__" }, { "n__" }]]]
		];  
		
		Notation[
			ParsedBoxWrapper[operatorBox["A_", operatorIndexBox[#1, "", "", True]&]] 
			\[DoubleLongLeftRightArrow]
			ParsedBoxWrapper[toBoxes[Operator["A_", "H"]]]
		];
		
		Notation[
			ParsedBoxWrapper[operatorBox["A_", operatorIndexBox[#1, "m__", "", True]&]] 
			\[DoubleLongLeftRightArrow]
			ParsedBoxWrapper[toBoxes[Operator["A_", "H", { "m__" }, {}]]]
		];  
		
		Notation[
			ParsedBoxWrapper[operatorBox["A_", operatorIndexBox[#1, "", "n__", True]&]] 
			\[DoubleLongLeftRightArrow]
			ParsedBoxWrapper[toBoxes[Operator["A_", "H", {}, { "n__" }]]]
		];  
		
		Notation[
			ParsedBoxWrapper[operatorBox["A_", operatorIndexBox[#1, "m__", "n__", True]&]] 
			\[DoubleLongLeftRightArrow]
			ParsedBoxWrapper[toBoxes[Operator["A_", "H", { "m__" }, { "n__" }]]]
		];
		
		
		(* Supplemental Notations *)
		Notation[\!\(\*
			TagBox[
			SubscriptBox[
			RowBox[{"[", 
			RowBox[{"A_", ",", "B_"}], "]"}], "-"],
			"NotationTemplateTag",
			BaseStyle->"NotationTemplateStyle"]\) \[DoubleLongLeftRightArrow] \
			\!\(\*
			TagBox[
			RowBox[{"Commutator", "[", 
			RowBox[{"A_", ",", "B_"}], "]"}],
			"NotationTemplateTag",
			BaseStyle->"NotationTemplateStyle"]\)
		];
		  
		Notation[\!\(\*
			TagBox[
			SuperscriptBox[
			TagBox[
			RowBox[{"f_", "?", "AtomQ"}],
			"NotationPatternTag",
			BaseStyle->"NotationPatternWrapperStyle"], "*"],
			"NotationTemplateTag",
			BaseStyle->"NotationTemplateStyle"]\) \[DoubleLongLeftRightArrow] \
			\!\(\*
			TagBox[
			RowBox[{"Conjugate", "[", 
			TagBox[
			RowBox[{"f_", "?", "AtomQ"}],
			"NotationPatternTag",
			BaseStyle->"NotationPatternWrapperStyle"], "]"}],
			"NotationTemplateTag",
			BaseStyle->"NotationTemplateStyle"]\)
		];
		
		Notation[\!\(\*
			TagBox[
			RowBox[{
			SuperscriptBox[
			RowBox[{"(", 
			TagBox[
			RowBox[{"f_", "?", 
			RowBox[{"(", 
			RowBox[{
			RowBox[{"!", 
			RowBox[{"AtomQ", "[", "#", "]"}]}], "&"}], ")"}]}],
			"NotationPatternTag",
			BaseStyle->"NotationPatternWrapperStyle"], ")"}], "*"], " "}],
			"NotationTemplateTag",
			BaseStyle->"NotationTemplateStyle"]\) \[DoubleLongLeftRightArrow] \
			\!\(\*
			TagBox[
			RowBox[{"Conjugate", "[", 
			TagBox[
			RowBox[{"f_", "?", 
			RowBox[{"(", 
			RowBox[{
			RowBox[{"!", 
			RowBox[{"AtomQ", "[", "#", "]"}]}], "&"}], ")"}]}],
			"NotationPatternTag",
			BaseStyle->"NotationPatternWrapperStyle"], "]"}],
			"NotationTemplateTag",
			BaseStyle->"NotationTemplateStyle"]\)
		];
		
		Notation[\!\(\*
			TagBox[
			SuperscriptBox[
			TagBox[
			RowBox[{"P_", "?", "OperatorQ"}],
			"NotationPatternTag",
			BaseStyle->"NotationPatternWrapperStyle"], 
			TagBox[
			RowBox[{"n_", "?", "Negative"}],
			"NotationPatternTag",
			BaseStyle->"NotationPatternWrapperStyle"]],
			"NotationTemplateTag",
			BaseStyle->"NotationTemplateStyle"]\) \[DoubleLongLeftRightArrow] \
			\!\(\*
			TagBox[
			RowBox[{"Power", "[", 
			RowBox[{
			TagBox[
			RowBox[{"P_", "?", "OperatorQ"}],
			"NotationPatternTag",
			BaseStyle->"NotationPatternWrapperStyle"], ",", 
			TagBox[
			RowBox[{"n_", "?", "Negative"}],
			"NotationPatternTag",
			BaseStyle->"NotationPatternWrapperStyle"]}], "]"}],
			"NotationTemplateTag",
			BaseStyle->"NotationTemplateStyle"]\)
		];
		
		Notation[\!\(\*
			TagBox[
			RowBox[{"c_", " ", 
			SuperscriptBox[
			TagBox[
			RowBox[{"P_", "?", "OperatorQ"}],
			"NotationPatternTag",
			BaseStyle->"NotationPatternWrapperStyle"], 
			TagBox[
			RowBox[{"n_", "?", "Negative"}],
			"NotationPatternTag",
			BaseStyle->"NotationPatternWrapperStyle"]]}],
			"NotationTemplateTag",
			BaseStyle->"NotationTemplateStyle"]\) \[DoubleLongLeftRightArrow] \
			\!\(\*
			TagBox[
			RowBox[{"Times", "[", 
			RowBox[{"c_", ",", 
			RowBox[{"Power", "[", 
			RowBox[{
			TagBox[
			RowBox[{"P_", "?", "OperatorQ"}],
			"NotationPatternTag",
			BaseStyle->"NotationPatternWrapperStyle"], ",", 
			TagBox[
			RowBox[{"n_", "?", "Negative"}],
			"NotationPatternTag",
			BaseStyle->"NotationPatternWrapperStyle"]}], "]"}]}], "]"}],
			"NotationTemplateTag",
			BaseStyle->"NotationTemplateStyle"]\)
		];

		InfixNotation[\!\(\*
			TagBox["\[CenterDot]",
			"NotationTemplateTag",
			BaseStyle->"NotationTemplateStyle"]\), NonCommutativeMultiply
		];
		
		SetQuantumNotationSupplement[];
		
		True
	]

SetQuantumNotation[]:=
	Module[
		{nb},
		nb := InputNotebook[];
		If[nb == $Failed,
			Return[False]];
		SetQuantumNotation[nb]
	];


(*	Notations for RulesSupplement package
*)
SetQuantumNotationSupplement[] :=
	Module[{},
		
		(* An Integral with one multidimensional integration variable *) 
		Notation[
			ParsedBoxWrapper[integralBox["ex_", "V_", "n_", "x_"]]
			\[DoubleLongLeftRightArrow] 
			ParsedBoxWrapper[toBoxes["QAIntegrate"["ex_", "V_", "n_", "x_"], "ex_"]]
		];		

		(* An Integral with two multidimensional integration variables of equal dimension *)
		Notation[
			ParsedBoxWrapper[integralBox["ex_", "V_", "n_", "x_", "y_"]]
			\[DoubleLongLeftRightArrow] 
			ParsedBoxWrapper[toBoxes["QAIntegrate"["ex_", "V_", "n_", "x_", "y_"], "ex_"]]
		];

		(* A Sum with one or more summation indices *)
		Notation[
			ParsedBoxWrapper[RowBox[{UnderscriptBox["\[Sum]", "seq__"], "ex_"}]] 
			\[DoubleLongLeftRightArrow] 
			ParsedBoxWrapper[RowBox[{"QASum", "[", RowBox[{"ex_", ",", "seq__"}], "]"}]]
		];

		(* A Product with one or more multiplication indices *)
		Notation[
			ParsedBoxWrapper[RowBox[{UnderscriptBox["\[Product]", "seq__"], "ex_"}]] 
			\[DoubleLongLeftRightArrow] 
			ParsedBoxWrapper[RowBox[{"QAProduct", "[", RowBox[{"ex_", ",", "seq__"}], "]"}]]
		];

		(* A Derivation with one or more derivation variables *)
		Notation[
			ParsedBoxWrapper[RowBox[{SubscriptBox["\[GothicCapitalD]", "seq__"], "ex_"}]] 
			\[DoubleLongLeftRightArrow] 
			ParsedBoxWrapper[RowBox[{"QAD", "[", RowBox[{"ex_", ",", "seq__"}], "]"}]]];

		(* A display notation for a Derivative operator *)
		Notation[
			ParsedBoxWrapper[SubscriptBox["\[PartialD]", "i_"]] 
			\[DoubleLongLeftArrow] 
			ParsedBoxWrapper[RowBox[{"QAD", "[", RowBox[{"Subscript", "[", RowBox[{"x", ",", "i_"}], "]"}], "]"}]]];

		
		(* Notation with Conversion *)
		Notation[
			ParsedBoxWrapper[SuperscriptBox["x_", "n__"]] 
			\[DoubleLongRightArrow] 
			ParsedBoxWrapper[toBoxes["ScriptConvert"["x_", "n__"]]]
		];

		Notation[
			ParsedBoxWrapper[SubsuperscriptBox["x_", "m__", "n__"]] 
			\[DoubleLongRightArrow] 
			ParsedBoxWrapper[toBoxes["ScriptConvert2"["x_", { "m__" }, { "n__" }]]]
		];
			
		(* Notations for Simplescripts *)
		Notation[
			ParsedBoxWrapper[SuperscriptBox["x_", "n_"]] 
			\[DoubleLongLeftArrow] 
			ParsedBoxWrapper[toBoxes["QASimplescript"["x_", "n_", "True"]]]
		];
			
		Notation[
			ParsedBoxWrapper[SubscriptBox["x_", "n_"]] 
			\[DoubleLongLeftArrow] 
			ParsedBoxWrapper[toBoxes["QASimplescript"["x_", "n_", "False"]]]
		];
			
		Notation[
			ParsedBoxWrapper[SubscriptBox["x_", "n_"]] 
			\[DoubleLongRightArrow] 
			ParsedBoxWrapper[toBoxes["QAGetSimplescript"["x_", "n_", "False"]]]
		];

		(* Notations for Subsuperscripts *)
		Notation[
			ParsedBoxWrapper[SuperscriptBox["x_", RowBox[{"m_", ",", "n__"}]]] 
			\[DoubleLongLeftArrow] 
			ParsedBoxWrapper[toBoxes["QASubsuperscript"["x_", {}, {"m_", "n__"}]]]
		];
			
		Notation[
			ParsedBoxWrapper[SubscriptBox["x_", RowBox[{"m_", ",", "n__"}]]] 
			\[DoubleLongLeftRightArrow] 
			ParsedBoxWrapper[toBoxes["QASubsuperscript"["x_", {"m_", "n__"}, {}]]]
		];

		Notation[
			ParsedBoxWrapper[SubsuperscriptBox["x_", "m__", "n__"]] 
			\[DoubleLongLeftArrow] 
			ParsedBoxWrapper[toBoxes["QASubsuperscript"["x_", { "m__" }, { "n__" }]]]
		];
		
		True
	];



(* ::Subsection:: *)
(* Input Aliases *)

(***** keyboard aliases [ESC]alias[ESC] *****)

ClearAliases[] :=
	Module[
		{nb},
 		nb := InputNotebook[];
		If[nb == $Failed,
			Return[False]];
		ClearAliases[nb]
	];

ClearAliases[doc_NotebookObject] :=
	SetOptions[doc,InputAliases->{}];

SetNotationAliases[args___]:=
	SetQuantumAliases[args];

SetQuantumAliases[]:=
	Module[
		{nb},
		nb := InputNotebook[];
		If[nb == $Failed,
			Return[False]];
		SetQuantumAliases[nb]
	];

(*
	Implementation Remarks
	- the first implementation used SetOptions to set InputAliases
	- in the meantime we use AddInputAlias on the notebook document
	- this works better together with Notation to define Bra-Ket-Notation
	- an extra notebook defining those Aliases and Notations is no longer
	  required. 
*)
SetQuantumAliases[doc_NotebookObject]:=
	Module[
		{aliases, seq},
		seq = Sequence[
			"NotationTemplateTag",
			BaseStyle->"NotationTemplateStyle"
		];
    	ClearAliases[];
    	
    	aliases = 
    		{	(*
				"on" -> "\[CenterDot]",
				"tp" -> "\[CircleTimes]",
				*)
					
				"com" -> TagBox[
					SubscriptBox[
					RowBox[{"[", "\[SelectionPlaceholder]", "]"}], "-"],
					"NotationTemplateTag",
					BaseStyle->"NotationTemplateStyle"],

				(* Operator Aliases *)
				"op" -> 
					ParsedBoxWrapper[operatorBox["\[SelectionPlaceholder]"]], 
					
				"opd" -> 
					ParsedBoxWrapper[operatorBox["\[SelectionPlaceholder]", operatorIndexBox[#1, "\[SelectionPlaceholder]", ""]&]], 
					
				"opu" -> 
					ParsedBoxWrapper[operatorBox["\[SelectionPlaceholder]", operatorIndexBox[#1, "", "\[SelectionPlaceholder]"]&]], 
					
				"opdu" -> 
					ParsedBoxWrapper[operatorBox["\[SelectionPlaceholder]", operatorIndexBox[#1, "\[SelectionPlaceholder]", "\[SelectionPlaceholder]"]&]], 
				
				
				"hop" -> 
					ParsedBoxWrapper[operatorBox["\[SelectionPlaceholder]", operatorIndexBox[#1, "", "", True]&]], 
				
				"hopd" -> 
					ParsedBoxWrapper[operatorBox["\[SelectionPlaceholder]", operatorIndexBox[#1, "\[SelectionPlaceholder]", "", True]&]], 
				
				"hopu" -> 
					ParsedBoxWrapper[operatorBox["\[SelectionPlaceholder]", operatorIndexBox[#1, "", "\[SelectionPlaceholder]", True]&]], 
				
				"hopdu" ->
					ParsedBoxWrapper[operatorBox["\[SelectionPlaceholder]", operatorIndexBox[#1, "\[SelectionPlaceholder]", "\[SelectionPlaceholder]", True]&]], 

				
				(* Ket Aliases *)
				"ket" -> 
					ParsedBoxWrapper[ketBox["\[SelectionPlaceholder]"]],				

				"ketd" -> 
					ParsedBoxWrapper[ketBox["\[SelectionPlaceholder]", ketIndexBox[#1, SubscriptBox, "\[SelectionPlaceholder]"]&]],
				
				"ketu" -> 
					ParsedBoxWrapper[ketBox["\[SelectionPlaceholder]", ketIndexBox[#1, SuperscriptBox, "\[SelectionPlaceholder]"]&]],
				
				"ketdu" -> 
					ParsedBoxWrapper[ketBox["\[SelectionPlaceholder]", ketIndexBox[#1, SubsuperscriptBox, "\[SelectionPlaceholder]", "\[SelectionPlaceholder]"]&]],

				(* Bra Aliases *)
				"bra" -> 
					ParsedBoxWrapper[braBox["\[SelectionPlaceholder]"]],				
				
				"dbra" -> 
					ParsedBoxWrapper[braBox["\[SelectionPlaceholder]", braIndexBox[#1, SubscriptBox, "\[SelectionPlaceholder]"]&]],
				
				"ubra" -> 
					ParsedBoxWrapper[braBox["\[SelectionPlaceholder]", braIndexBox[#1, SuperscriptBox, "\[SelectionPlaceholder]"]&]],
				
				"dubra" -> 
					ParsedBoxWrapper[braBox["\[SelectionPlaceholder]", braIndexBox[#1, SubsuperscriptBox, "\[SelectionPlaceholder]", "\[SelectionPlaceholder]"]&]],


				(* BraKet Aliases *)
				"braket" -> 
					ParsedBoxWrapper[braKetBox["\[SelectionPlaceholder]", "\[SelectionPlaceholder]"]], 
								
				"braketd" -> 
					ParsedBoxWrapper[braKetBox["\[SelectionPlaceholder]", "\[SelectionPlaceholder]", ketIndexBox[#1, SubscriptBox, "\[SelectionPlaceholder]"]&]],
				
				"braketu" -> 
					ParsedBoxWrapper[braKetBox["\[SelectionPlaceholder]", "\[SelectionPlaceholder]", ketIndexBox[#1, SuperscriptBox, "\[SelectionPlaceholder]"]&]],
				
				"braketdu" -> 
					ParsedBoxWrapper[braKetBox["\[SelectionPlaceholder]", "\[SelectionPlaceholder]", ketIndexBox[#1, SubsuperscriptBox, "\[SelectionPlaceholder]", "\[SelectionPlaceholder]"]&]],
				
				"dbraket" -> 
					ParsedBoxWrapper[braKetBox["\[SelectionPlaceholder]", "\[SelectionPlaceholder]", braIndexBox[#1, SubscriptBox, "\[SelectionPlaceholder]"]&]],
				
				"ubraket" -> 
					ParsedBoxWrapper[braKetBox["\[SelectionPlaceholder]", "\[SelectionPlaceholder]", braIndexBox[#1, SuperscriptBox, "\[SelectionPlaceholder]"]&]],
				
				"dubraket" -> 
					ParsedBoxWrapper[braKetBox["\[SelectionPlaceholder]", "\[SelectionPlaceholder]", braIndexBox[#1, SubsuperscriptBox, "\[SelectionPlaceholder]", "\[SelectionPlaceholder]"]&]],
				
				"dbraketd" -> 
					ParsedBoxWrapper[braKetBox["\[SelectionPlaceholder]", "\[SelectionPlaceholder]", 
						braIndexBox[#1, SubscriptBox, "\[SelectionPlaceholder]"]&, 
						ketIndexBox[#1, SubscriptBox, "\[SelectionPlaceholder]"]&]],
				
				"dbraketu" -> 
					ParsedBoxWrapper[braKetBox["\[SelectionPlaceholder]", "\[SelectionPlaceholder]", 
						braIndexBox[#1, SubscriptBox, "\[SelectionPlaceholder]"]&, 
						ketIndexBox[#1, SuperscriptBox, "\[SelectionPlaceholder]"]&]],
				
				"dbraketdu" -> 
					ParsedBoxWrapper[braKetBox["\[SelectionPlaceholder]", "\[SelectionPlaceholder]", 
						braIndexBox[#1, SubscriptBox, "\[SelectionPlaceholder]"]&, 
						ketIndexBox[#1, SubsuperscriptBox, "\[SelectionPlaceholder]", "\[SelectionPlaceholder]"]&]],
				
				"ubraketd" -> 
					ParsedBoxWrapper[braKetBox["\[SelectionPlaceholder]", "\[SelectionPlaceholder]", 
						braIndexBox[#1, SuperscriptBox, "\[SelectionPlaceholder]"]&, 
						ketIndexBox[#1, SubscriptBox, "\[SelectionPlaceholder]"]&]],
				
				"ubraketu" -> 
					ParsedBoxWrapper[braKetBox["\[SelectionPlaceholder]", "\[SelectionPlaceholder]", 
						braIndexBox[#1, SuperscriptBox, "\[SelectionPlaceholder]"]&, 
						ketIndexBox[#1, SuperscriptBox, "\[SelectionPlaceholder]"]&]],
				
				"ubraketdu" -> 
					ParsedBoxWrapper[braKetBox["\[SelectionPlaceholder]", "\[SelectionPlaceholder]", 
						braIndexBox[#1, SuperscriptBox, "\[SelectionPlaceholder]"]&, 
						ketIndexBox[#1, SubsuperscriptBox, "\[SelectionPlaceholder]", "\[SelectionPlaceholder]"]&]],
				
				"dubraketd" -> 
					ParsedBoxWrapper[braKetBox["\[SelectionPlaceholder]", "\[SelectionPlaceholder]", 
						braIndexBox[#1, SubsuperscriptBox, "\[SelectionPlaceholder]", "\[SelectionPlaceholder]"]&, 
						ketIndexBox[#1, SubscriptBox, "\[SelectionPlaceholder]"]&]],
				
				"dubraketu" -> 
					ParsedBoxWrapper[braKetBox["\[SelectionPlaceholder]", "\[SelectionPlaceholder]", 
						braIndexBox[#1, SubsuperscriptBox, "\[SelectionPlaceholder]", "\[SelectionPlaceholder]"]&, 
						ketIndexBox[#1, SuperscriptBox, "\[SelectionPlaceholder]"]&]],
				
				"dubraketdu" -> 
					ParsedBoxWrapper[braKetBox["\[SelectionPlaceholder]", "\[SelectionPlaceholder]", 
						braIndexBox[#1, SubsuperscriptBox, "\[SelectionPlaceholder]", "\[SelectionPlaceholder]"]&, 
						ketIndexBox[#1, SubsuperscriptBox, "\[SelectionPlaceholder]", "\[SelectionPlaceholder]"]&]],

				
				(* Aliases for Script notation *)
				"sd" -> ParsedBoxWrapper[SubscriptBox["\[SelectionPlaceholder]", "\[SelectionPlaceholder]"]],
					
				"su" -> ParsedBoxWrapper[SuperscriptBox["\[SelectionPlaceholder]", "\[SelectionPlaceholder]"]],
					
				"sdu" -> ParsedBoxWrapper[SubsuperscriptBox["\[SelectionPlaceholder]", "\[SelectionPlaceholder]", "\[SelectionPlaceholder]"]],
				
				
				(* Aliases for Integrals and Sums *)
				"qint" -> ParsedBoxWrapper[integralBox["\[SelectionPlaceholder]", "\[SelectionPlaceholder]", "\[SelectionPlaceholder]", "\[SelectionPlaceholder]"]],

				"qint2" -> ParsedBoxWrapper[integralBox["\[SelectionPlaceholder]", "\[SelectionPlaceholder]", "\[SelectionPlaceholder]", "\[SelectionPlaceholder]", "\[SelectionPlaceholder]"]],
				
				"qsum" -> ParsedBoxWrapper[RowBox[{UnderscriptBox["\[Sum]", "\[SelectionPlaceholder]"], "\[SelectionPlaceholder]"}]],
				
				"qd" -> ParsedBoxWrapper[RowBox[{SubscriptBox["\[GothicCapitalD]", "\[SelectionPlaceholder]"], "\[SelectionPlaceholder]"}]]
    		};

		Map[
			AddInputAlias[
				If[MatchQ[#, Rule[_, _TagBox]],
				# /. 
					((alias_ -> cbox: TagBox[box_, tag_, opt___]) -> 
					(alias -> ParsedBoxWrapper[box])),
				#],
				doc]&, 
			aliases];

		SetQuantumAliases::aliases
	];

(* ::Subsection:: *)
(* SetBoxes *)

SetBoxes[] :=
	Module[{nb},
		nb = InputNotebook[];
		If[nb == $Failed,
			Return[False]];
		Unprotect@SubscriptBox;
		SubscriptBox[RowBox[{x_, "[", RowBox[y_], "]"}], "-"] := 
			RowBox[{x, " ", SubscriptBox[RowBox[{"[", RowBox[y], "]"}], "-"]}];
		Protect@SubscriptBox;
		Unprotect@RowBox;
		RowBox[{"(", TagBox[arg_, Ket, opts___], ")"}] := 
			TagBox[arg, Ket, opts];
		RowBox[{"(", TagBox[arg_, Bra, opts___], ")"}] := 
			TagBox[arg, Bra, opts];
		RowBox[{"(", TagBox[arg_, BraKet, opts___], ")"}] := 
			TagBox[ arg, BraKet, opts];
		Protect@RowBox;
		Unprotect@FullForm;
		FullForm /: TagBox[StyleBox[expr_, Verbatim[ShowSpecialCharacters -> False], rest__], FullForm] := 
			TagBox[StyleBox[expr, rest], FullForm];
		Protect@FullForm;	
		True	
	]

(* Closes the Notation Palette 
*)
CloseNotationPalette[] :=
	Module[{np},
		np = Notebooks["Notation Palette"];
		If[Length[np] == 1,
			NotebookClose[np // First]
		];
		True
	]
	

(* ::Subsection:: *)
(* Internal *)

(* Returns a vertical separator as adjustment box and configured by its margin *)
verticalSeparator[margins_List] :=
	Module[{},
		AdjustmentBox["\[VerticalSeparator]", BoxMargins->{ margins, {0, 0}}]	
	]
	
(* Returns a blank vertical separator *)
verticalSeparator[] :=
	Module[{},
		"\[VerticalSeparator]"	
	]

(* Returns a right angle bracket *)
rightBracket[] := "\[RightAngleBracket]"	

(* Returns a left angle bracket *)
leftBracket[] := "\[LeftAngleBracket]"	
	
(* Excloses the argument(s) of an object in boxes *)
argument[args_String, argtype_Symbol] :=
	Module[{},
		AdjustmentBox[
			TagBox[args, argtype, BaseStyle->"BraKetArg"],
			BoxBaselineShift->0]
	]

(* Generates a ket box, optionally accompanied by indices *)
ketBox[args_String, func_Function : (#1&)] :=
	Module[{rowBox, res},
		rowBox[] := RowBox[
			{
				verticalSeparator[{-0.4, 0.0}],
				argument[args, KetArgs], 
				rightBracket[]
			}];
		
		res = TagBox[
			func[rowBox[]],
			Ket,
			BaseStyle->"KetWrapper"
		];
		res
	]

(* Generates a bra box, optionally accompanied by indices *)
braBox[args_String, func_Function : (#1&)] :=
	Module[{rowBox},
		rowBox[] := RowBox[
			{
				leftBracket[],
				argument[args, BraArgs], 
				verticalSeparator[{0.0, -0.2}]
			}];
		
		TagBox[
			func[rowBox[]],
			Bra,
			BaseStyle->"BraWrapper"
		]
	]

(* Generates a braket box, optionally accompanied by indices *)
braKetBox[largs_String, rargs_String, ofunc_Function : (#1&), ifunc_Function : (#1&)] :=
	Module[{rowBox},
		rowBox[] := RowBox[
			{
				leftBracket[], 
				AdjustmentBox[
					RowBox[{
						TagBox[largs, BraKetArgs, BaseStyle->"BraKetArg"], 
						verticalSeparator[], 
						TagBox[rargs, BraKetArgs, BaseStyle->"BraKetArg"]
					}],
					BoxBaselineShift->0
				], 
				rightBracket[]
			}];
			
		TagBox[
			ofunc[ifunc[rowBox[]]],
			BraKet,
			BaseStyle->"BraKetWrapper"
		]	
	]
	
(* Generates an operator box, optionally accompanied by indices and / or super-dagger *)
operatorBox[arg_String, func_Function : (#1&)] := 
	Module[{},
		func[OverscriptBox[arg, "^"]]
	]
	
(* Generates all kind of indices for the operator box by injecting it into the box invocation *)
operatorIndexBox[baseBox_, lindices_String, uindices_String, dagger: (True | False) : False] :=
	Module[{box, upper},
		upper = Piecewise[{
			{ RowBox[{uindices, ",", "\[Dagger]"}], dagger && uindices != "" },
			{ "\[Dagger]", dagger },
			{ uindices, uindices != "" }
		}];
		
		box = Piecewise[{
			{ SubsuperscriptBox[baseBox, lindices, upper], lindices != "" && upper =!= 0 },
			{ SubscriptBox[baseBox, lindices], lindices != "" },
			{ SuperscriptBox[baseBox, upper], upper =!= 0 }},
			baseBox];
		box
	]

(* Generates an index, the position of which is defined by style and the using function *)
indexBox[args_String, argtype_, style_String] :=
	Module[{},
		TagBox[
			argument[args, argtype], 
			UDScript,
			BaseStyle->style
		]
	]

(* Generates "Ket" indices, i.e. indices on the right hand side of objects *)
ketIndexBox[baseBox_, scriptBox_, indices__String] :=
	Module[{},
		scriptBox[baseBox, Sequence@@Map[indexBox[#1, KetArgs, "RIndex"]&, { indices }]]
	]

(* Generates "Bra" indices, i.e. indices on the left hand side of objects *)
braIndexBox[baseBox_, scriptBox_, indices__String] :=
	Module[{},
		RowBox[{
			scriptBox["\[InvisiblePrefixScriptBase]", Sequence@@Map[indexBox[#1, BraArgs, "LIndex"]&, { indices }]],
			baseBox					
		}]	
	]

integralBox[ex_String, V_String, n_String, x__String] := 
	Module[{},
		RowBox[{
			SubscriptBox["\[Integral]", V], 
			RowBox[{
				ex, 
				Sequence@@Map[RowBox[{SuperscriptBox["\[DifferentialD]", n], #}]&, { x }]
			}]
		}]
	]

(*	Common function to convert an object representation to box form as is understood by Notation
*)
toBoxes[object_, wrap_String : ""] :=
	Module[{traditionalForm, first},
		traditionalForm = ToBoxes[object, TraditionalForm];								(* TraditionalForm is form without extra notation *)
		first = First[List @@ traditionalForm];											(* We extract the part we need here *)
		first = first /. {"(" -> "[", ")" -> "]"};										(* It has parenthesis -> replace by brackets *)
		first = first /. { s_String :> StringReplace[s, "\"" ~~ x___ ~~ "\"" -> x] }; 	(* Remove the extra quotes generated because object already contains them *) 
		first = first /. { s_String :> StringReplace[s, "H" -> "\"\\<H\\>\""] };		(* Parsing form for "H" required *)
		first = first /. wrap -> RowBox[{"(", wrap, ")"}];								(* wrap indicates an optional expression to be wrapped *)
		first
	]
	

End[]

EndPackage[]
